@@ -43,6 +43,15 @@
  * a value obtained from Sofia-SIP documentation */
 #define SIP_CONNECTION_DEFAULT_KEEPALIVE_INTERVAL 120
 
+/* The user is not allowed to set keepalive timeout to lower than that,
+ * to avoid wasting traffic and device power */
+#define SIP_CONNECTION_MINIMUM_KEEPALIVE_INTERVAL 30
+
+/* The user is not allowed to set keepalive timeout to lower than that
+ * for REGISTER keepalives, to avoid wasting traffic and device power.
+ * REGISTER is special because it may tie resources on the server side */
+#define SIP_CONNECTION_MINIMUM_KEEPALIVE_INTERVAL_REGISTER 50
+
 /* The value of SIP_NH_EXPIRED. This can be anything that is neither NULL
  * nor a media channel */
 NUA_HMAGIC_T * const _sip_nh_expired = (NUA_HMAGIC_T *)"";
@@ -131,6 +140,92 @@ sip_conn_save_event (SIPConnection *conn,
   nua_save_event (priv->sofia_nua, ret_saved);
 }
 
+void
+sip_conn_update_proxy_and_transport (SIPConnection *conn)
+{
+  SIPConnectionPrivate *priv = SIP_CONNECTION_GET_PRIVATE (conn);
+
+  if (priv->proxy_url != NULL)
+    {
+      su_home_t temphome[1] = { SU_HOME_INIT(temphome) };
+      sip_route_t *route;
+      url_t *route_url;
+      const char *params = NULL;
+
+      route_url = url_hdup (temphome, priv->proxy_url);
+
+      g_return_if_fail (route_url != NULL);
+
+      if (!url_has_param (route_url, "lr"))
+        url_param_add (temphome, route_url, "lr");
+
+      route = sip_route_create (temphome, route_url, NULL);
+
+      if (priv->transport != NULL && route_url->url_type == url_sip)
+        {
+          if (g_ascii_strcasecmp (priv->transport, "tcp") == 0)
+            params = "transport=tcp";
+          else if (g_ascii_strcasecmp (priv->transport, "udp") == 0)
+            params = "transport=udp";
+          else
+            g_warning ("unrecognized transport parameter value: %s", priv->transport);
+        }
+
+      nua_set_params (priv->sofia_nua,
+                      NUTAG_INITIAL_ROUTE(route),
+                      TAG_IF(params, NUTAG_M_PARAMS(params)),
+                      TAG_NULL());
+
+      su_home_deinit (temphome);
+    }
+}
+
+const url_t *
+sip_conn_get_local_url (SIPConnection *conn)
+{
+  SIPConnectionPrivate *priv = SIP_CONNECTION_GET_PRIVATE (conn);
+  url_t *url;
+
+  url = url_make (priv->sofia_home, "sip:*:*");
+
+  if (url == NULL)
+    return NULL;
+
+  if (priv->proxy_url != NULL)
+    {
+      url->url_type = priv->proxy_url->url_type;
+    }
+  else
+    {
+      g_assert (priv->account_url != NULL);
+      url->url_type = priv->account_url->url_type;
+    }
+
+  if (priv->local_ip_address == NULL)
+    url->url_host = "*";
+  else
+    url->url_host = priv->local_ip_address;
+
+  if (priv->local_port == 0)
+    url->url_port = "*";
+  else
+    url->url_port = su_sprintf (priv->sofia_home, "%u", priv->local_port);
+
+  if (url->url_type == url_sip && priv->transport != NULL)
+    {
+      if (!g_ascii_strcasecmp(priv->transport, "udp"))
+        url->url_params = "transport=udp";
+      else if (!g_ascii_strcasecmp(priv->transport, "tcp"))
+        url->url_params = "transport=tcp";
+    }
+
+  /* url_sanitize (url); -- we're always sane B-] */
+
+  DEBUG("local binding expressed as <" URL_PRINT_FORMAT ">", URL_PRINT_ARGS(url));
+
+  return url;
+}
+
 static GHashTable*
 priv_nua_get_outbound_options (nua_t* nua)
 {
@@ -264,6 +359,9 @@ sip_conn_update_nua_outbound (SIPConnection *conn)
       break;
     }
 
+  g_hash_table_insert (option_table,
+                       g_strdup ("natify"),
+                       GINT_TO_POINTER(priv->discover_binding));
   g_hash_table_insert (option_table,
                        g_strdup ("use-rport"),
                        GINT_TO_POINTER(priv->discover_binding));
@@ -275,6 +373,24 @@ sip_conn_update_nua_outbound (SIPConnection *conn)
   g_hash_table_destroy (option_table);
 }
 
+static void
+priv_sanitize_keepalive_interval (SIPConnectionPrivate *priv)
+{
+  gint minimum_interval;
+  if (priv->keepalive_interval > 0)
+    {
+      minimum_interval =
+              (priv->keepalive_mechanism == SIP_CONNECTION_KEEPALIVE_REGISTER)
+              ? SIP_CONNECTION_MINIMUM_KEEPALIVE_INTERVAL_REGISTER
+              : SIP_CONNECTION_MINIMUM_KEEPALIVE_INTERVAL;
+      if (priv->keepalive_interval < minimum_interval)
+        {
+          g_warning ("keepalive interval is too low, pushing to %d", minimum_interval);
+          priv->keepalive_interval = minimum_interval;
+        }
+    }
+}
+
 void
 sip_conn_update_nua_keepalive_interval (SIPConnection *conn)
 {
@@ -287,7 +403,10 @@ sip_conn_update_nua_keepalive_interval (SIPConnection *conn)
     /* XXX: figure out proper default timeouts depending on transport */
     keepalive_interval = SIP_CONNECTION_DEFAULT_KEEPALIVE_INTERVAL;
   else
-    keepalive_interval = (long)priv->keepalive_interval;
+    {
+      priv_sanitize_keepalive_interval (priv);
+      keepalive_interval = (long) priv->keepalive_interval;
+    }
   keepalive_interval *= 1000;
 
   DEBUG("setting keepalive interval to %ld msec", keepalive_interval);
@@ -307,9 +426,10 @@ sip_conn_update_nua_contact_features (SIPConnection *conn)
   if (priv->keepalive_mechanism != SIP_CONNECTION_KEEPALIVE_REGISTER)
     return;
 
+  priv_sanitize_keepalive_interval (priv);
   timeout = (priv->keepalive_interval > 0)
-	? priv->keepalive_interval
-	: SIP_CONNECTION_DEFAULT_KEEPALIVE_INTERVAL;
+        ? priv->keepalive_interval
+        : SIP_CONNECTION_DEFAULT_KEEPALIVE_INTERVAL;
   contact_features = g_strdup_printf ("expires=%u", timeout);
   nua_set_params(priv->sofia_nua,
 		 NUTAG_M_FEATURES(contact_features),
