@@ -12,6 +12,11 @@
  * modify it under the terms of the GNU Lesser General Public License
  * version 2.1 as published by the Free Software Foundation.
  *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
@@ -47,12 +52,13 @@ typedef struct {
     guint port;
     gchar *transport;
     gboolean discover_binding;
-    gboolean use_http_proxy;
     gchar *keepalive_mechanism;
     gint keepalive_interval;
     gboolean discover_stun;
     gchar *stun_server;
     guint stun_port;
+    gchar *local_ip_address;
+    guint local_port;
     gchar *extra_auth_user;
     gchar *extra_auth_password;
     gboolean avoid_difficult;
@@ -77,6 +83,7 @@ free_params (void *p)
   g_free (params->transport);
   g_free (params->keepalive_mechanism);
   g_free (params->stun_server);
+  g_free (params->local_ip_address);
   g_free (params->extra_auth_user);
   g_free (params->extra_auth_password);
 
@@ -92,12 +99,13 @@ enum {
     SIP_CONN_PARAM_PORT,
     SIP_CONN_PARAM_TRANSPORT,
     SIP_CONN_PARAM_DISCOVER_BINDING,
-    SIP_CONN_PARAM_USE_HTTP_PROXY,
     SIP_CONN_PARAM_KEEPALIVE_MECHANISM,
     SIP_CONN_PARAM_KEEPALIVE_INTERVAL,
     SIP_CONN_PARAM_DISCOVER_STUN,
     SIP_CONN_PARAM_STUN_SERVER,
     SIP_CONN_PARAM_STUN_PORT,
+    SIP_CONN_PARAM_LOCAL_IP_ADDRESS,
+    SIP_CONN_PARAM_LOCAL_PORT,
     SIP_CONN_PARAM_EXTRA_AUTH_USER,
     SIP_CONN_PARAM_EXTRA_AUTH_PASSWORD,
     SIP_CONN_PARAM_AVOID_DIFFICULT,
@@ -132,15 +140,11 @@ static const TpCMParamSpec sip_params[] = {
     { "transport", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING,
       TP_CONN_MGR_PARAM_FLAG_HAS_DEFAULT, "auto",
       G_STRUCT_OFFSET (SIPConnParams, transport) },
-    /* Not used */
+    /* Used to enable proactive NAT traversal techniques */
     { "discover-binding", DBUS_TYPE_BOOLEAN_AS_STRING, G_TYPE_BOOLEAN,
       TP_CONN_MGR_PARAM_FLAG_HAS_DEFAULT, GUINT_TO_POINTER(TRUE),
       G_STRUCT_OFFSET (SIPConnParams, discover_binding) },
-    /* Not used */
-    { "use-http-proxy", DBUS_TYPE_BOOLEAN_AS_STRING, G_TYPE_BOOLEAN,
-      TP_CONN_MGR_PARAM_FLAG_HAS_DEFAULT, GUINT_TO_POINTER(FALSE),
-      G_STRUCT_OFFSET (SIPConnParams, use_http_proxy) },
-    /* Not used */
+    /* Mechanism used for connection keepalive maintenance */
     { "keepalive-mechanism", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING,
       0, NULL, G_STRUCT_OFFSET (SIPConnParams, keepalive_mechanism) },
     /* KA interval */
@@ -158,6 +162,12 @@ static const TpCMParamSpec sip_params[] = {
       TP_CONN_MGR_PARAM_FLAG_HAS_DEFAULT,
       GUINT_TO_POINTER(SIP_DEFAULT_STUN_PORT),
       G_STRUCT_OFFSET (SIPConnParams, stun_port) },
+    /* Local IP address to use, workaround purposes only */
+    { "local-ip-address", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING,
+      0, NULL, G_STRUCT_OFFSET (SIPConnParams, local_ip_address) },
+    /* Local port for SIP, workaround purposes only */
+    { "local-port", DBUS_TYPE_UINT16_AS_STRING, G_TYPE_UINT,
+      0, NULL, G_STRUCT_OFFSET (SIPConnParams, local_port) },
     /* Extra authentication */
     { "extra-auth-user", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING,
       0, NULL, G_STRUCT_OFFSET (SIPConnParams, extra_auth_user) },
@@ -238,32 +248,21 @@ priv_compose_proxy_uri (const gchar *host,
                         guint port)
 {
   const gchar *scheme = "sip";
-  const gchar *params = "";
 
   if (host == NULL)
     return NULL;
 
-  /* Encode transport */
+  /* Set scheme to SIPS if transport is TLS */
 
-  if (transport == NULL || !strcmp (transport, "auto")) {
-    /*no mention of transport in the URI*/
-  } else if (!strcmp (transport, "tcp")) {
-    params = ";transport=tcp";
-  } else if (!strcmp (transport, "udp")) {
-    params = ";transport=udp";
-  } else if (!strcmp (transport, "tls")) {
+  if (transport != NULL && !g_ascii_strcasecmp (transport, "tls"))
     scheme = "sips";
-  } else {
-    g_warning ("transport %s not recognized", transport);
-  }
 
   /* Format the resulting URI */
 
-  if (port) {
-    return g_strdup_printf ("%s:%s:%u%s", scheme, host, port, params);
-  } else {
-    return g_strdup_printf ("%s:%s%s", scheme, host, params);
-  }
+  if (port)
+    return g_strdup_printf ("%s:%s:%u", scheme, host, port);
+  else
+    return g_strdup_printf ("%s:%s", scheme, host);
 }
 
 /**
@@ -286,15 +285,19 @@ priv_compose_default_proxy_uri (const gchar *sip_address,
   /* skip sip and sips prefixes, updating transport if necessary */
   found = strchr (sip_address, ':');
   if (found != NULL) {
-    if (strncmp("sip:", sip_address, 4) == 0) {
+    if (g_ascii_strncasecmp ("sip:", sip_address, 4) == 0)
       ;
-    } else if (strncmp("sips:", sip_address, 5) == 0) {
-      if (transport == NULL || strcmp (transport, "auto") == 0)
-        transport = "tls";
-    } else {
-      /* error, unknown URI prefix */
-      return NULL;
-    }
+    else if (g_ascii_strncasecmp ("sips:", sip_address, 5) == 0)
+      {
+        if (transport == NULL ||
+            strcmp (transport, "auto") == 0)
+          transport = "tls";
+      }
+    else
+      {
+        /* error, unknown URI prefix */
+        return NULL;
+      }
 
     sip_address = found + 1;
   }
@@ -307,8 +310,10 @@ priv_compose_default_proxy_uri (const gchar *sip_address,
   /* copy rest of the string */
   host = g_strdup (sip_address);
 
-  /* mark end (uri-parameters defs) */
+  /* mark end (before uri-parameters defs or headers) */
   found = strchr (host, ';');
+  if (found == NULL)
+    found = strchr (host, '?');
   if (found != NULL)
     *found = '\0';
 
@@ -383,9 +388,6 @@ sip_connection_manager_new_connection (TpBaseConnectionManager *base,
     return NULL;
   }
 
-  /* TODO: retrieve global HTTP proxy settings if
-   * "use-http-proxy" parameter is enabled */
-
   /* TpBaseConnectionManager code has already checked that required params
    * are present (but not that they are non-empty, if we're using >= 0.5.8)
    */
@@ -413,6 +415,10 @@ sip_connection_manager_new_connection (TpBaseConnectionManager *base,
   if (!check_not_empty_if_present ("stun-server", params->stun_server,
         error))
     return FALSE;
+  /* FIXME: validate local IP address properly */
+  if (!check_not_empty_if_present ("local-ip-address", params->local_ip_address,
+        error))
+    return FALSE;
 
   DEBUG("New SIP connection to %s", params->account);
 
@@ -434,6 +440,9 @@ sip_connection_manager_new_connection (TpBaseConnectionManager *base,
   g_object_set (connection, "proxy", proxy, NULL);
   g_free (proxy);
 
+  if (params->transport != NULL && strcmp (params->transport, "auto") != 0)
+    g_object_set (connection, "transport", params->transport, NULL);
+
   SET_PROPERTY_IF_PARAM_SET ("auth-user", SIP_CONN_PARAM_AUTH_USER,
       params->auth_user);
 
@@ -461,6 +470,12 @@ sip_connection_manager_new_connection (TpBaseConnectionManager *base,
   keepalive_mechanism = priv_parse_keepalive (params->keepalive_mechanism);
   g_object_set (connection, "keepalive-mechanism", keepalive_mechanism, NULL);
 
+  SET_PROPERTY_IF_PARAM_SET ("local-ip-address", SIP_CONN_PARAM_LOCAL_IP_ADDRESS,
+      params->local_ip_address);
+
+  SET_PROPERTY_IF_PARAM_SET ("local-port", SIP_CONN_PARAM_LOCAL_PORT,
+      params->local_port);
+
   SET_PROPERTY_IF_PARAM_SET ("extra-auth-user", SIP_CONN_PARAM_EXTRA_AUTH_USER,
       params->extra_auth_user);
   SET_PROPERTY_IF_PARAM_SET ("extra-auth-password", SIP_CONN_PARAM_EXTRA_AUTH_PASSWORD,
