@@ -1,7 +1,7 @@
-/* 
- * Copyright (C) 2005-2007 Collabora Ltd. and Nokia Corporation
- *
- * sip-connection-private.h- Private structues for SIPConnection
+/*
+ * sip-connection-private.h - Private structures for SIPConnection
+ * Copyright (C) 2005-2007 Collabora Ltd.
+ * Copyright (C) 2005-2007 Nokia Corporation
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -11,10 +11,10 @@
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
+ *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the 
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA. 
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #ifndef __SIP_CONNECTION_PRIVATE_H__
@@ -32,9 +32,12 @@ struct _SIPConnectionPrivate
   nua_handle_t *register_op;
   sres_resolver_t *sofia_resolver;
   url_t *account_url;
+  url_t *proxy_url;
+  url_t *registrar_url;
 
   gchar *registrar_realm;
-  gchar *last_auth;
+
+  GHashTable *auth_table;
 
   /* channels */
   TpChannelFactoryIface *text_factory;
@@ -43,20 +46,19 @@ struct _SIPConnectionPrivate
   gchar *address;
   gchar *auth_user;
   gchar *password;
-  gchar *proxy;
-  gchar *registrar;
+  gchar *transport;
   SIPConnectionKeepaliveMechanism keepalive_mechanism;
   gint keepalive_interval;
-  gchar *http_proxy;
   gboolean discover_stun;
   gchar *stun_host;
   guint stun_port;
+  gchar *local_ip_address;
+  guint local_port;
   gchar *extra_auth_user;
   gchar *extra_auth_password;
   gboolean discover_binding;
 
-  gboolean dispose_has_run : 1;
-  gboolean register_succeeded : 1;
+  gboolean dispose_has_run;
 };
 
 #define SIP_PROTOCOL_STRING               "sip"
