@@ -66,8 +66,6 @@ G_DEFINE_TYPE_WITH_CODE(SIPConnection, sip_connection,
       return; \
     }
 
-static GObjectClass *parent_class=NULL;
-
 
 /* properties */
 enum
@@ -76,77 +74,50 @@ enum
   PROP_AUTH_USER,        /**< account username (if different from public address userinfo part) */
   PROP_PASSWORD,         /**< account password (for registration) */
 
+  PROP_TRANSPORT,        /**< outbound transport */
   PROP_PROXY,            /**< outbound SIP proxy (SIP URI) */
   PROP_REGISTRAR,        /**< SIP registrar (SIP URI) */
 
   PROP_KEEPALIVE_MECHANISM, /**< keepalive mechanism as defined by SIPConnectionKeepaliveMechanism */
   PROP_KEEPALIVE_INTERVAL, /**< keepalive interval in seconds */
-  PROP_HTTP_PROXY,         /**< HTTP proxy URI; use HTTP-CONNECT to reach SIP servers */
   PROP_DISCOVER_BINDING,   /**< enable discovery of public binding */
   PROP_DISCOVER_STUN,      /**< Discover STUN server name using DNS SRV lookup */
   PROP_STUN_SERVER,        /**< STUN server address (if not set, derived
 			        from public SIP address */
   PROP_STUN_PORT,          /**< STUN port */
+  PROP_LOCAL_IP_ADDRESS,   /**< Local IP address (normally not needed, chosen by stack) */
+  PROP_LOCAL_PORT,         /**< Local port for SIP (normally not needed, chosen by stack) */
   PROP_EXTRA_AUTH_USER,	   /**< User name to use for extra authentication challenges */
   PROP_EXTRA_AUTH_PASSWORD,/**< Password to use for extra authentication challenges */
   PROP_SOFIA_ROOT,         /**< Event root pointer from the Sofia-SIP stack */
   LAST_PROPERTY
 };
 
-/**
- * Returns a duplicated char array of a *user-provided* SIP 
- * URI, which might be missing the URI prefix, or some other 
- * required components.
- *
- * @return NULL if a valid URI cannot be created from 'input'
- */
-static gchar *priv_sip_strdup(const gchar *input)
+
+static void
+priv_value_set_url_as_string (GValue *value, const url_t *url)
 {
-  if (input == NULL)
-    return NULL;
-
-  /* Pretty relaxed, need a stronger solution */
-  if (strchr (input, ':'))
-    return g_strdup (input);
-
-  return g_strdup_printf ("sip:%s", input);
+  if (url == NULL)
+    {
+      g_value_set_string (value, NULL);
+    }
+  else
+    {
+      su_home_t temphome[1] = { SU_HOME_INIT(temphome) };
+      char *tempstr;
+      tempstr = url_as_string (temphome, url);
+      g_value_set_string (value, tempstr);
+      su_home_deinit (temphome);
+    }
 }
 
-static GObject *
-sip_connection_constructor (GType                  type,
-                            guint                  n_construct_properties,
-                            GObjectConstructParam *construct_properties)
+static url_t *
+priv_url_from_string_value (su_home_t *home, const GValue *value)
 {
-  GObject *obj;
-
-  {
-    /* Invoke parent constructor.
-     * this calls our init, and then set_property with any
-     * CONSTRUCT params
-     */
-    SIPConnectionClass *klass;
-    klass = SIP_CONNECTION_CLASS (g_type_class_peek (SIP_TYPE_CONNECTION));
-    parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
-    obj = parent_class->constructor (type,
-                                     n_construct_properties,
-                                     construct_properties);
-  }
-
-  SIPConnection *self = SIP_CONNECTION (obj);
-  SIPConnectionPrivate *priv = SIP_CONNECTION_GET_PRIVATE (self); 
-
-  /* step: create home objects */
-  priv->sofia_home = su_home_new(sizeof (su_home_t));
-
-  /* the non-construct parameters will be empty */
-  g_assert (priv->registrar == NULL);
-  g_assert (priv->proxy == NULL);
-  g_assert (priv->http_proxy == NULL);
-
-  g_assert (priv->sofia_nua == NULL);
-
-  g_message ("SIPConnection constructed at %p", obj);
-  return obj;
+  const gchar *url_str;
+  g_assert (home != NULL);
+  url_str = g_value_get_string (value);
+  return (url_str)? url_make (home, url_str) : NULL;
 }
 
 /* keep these two in sync */
@@ -209,6 +180,11 @@ sip_connection_init (SIPConnection *obj)
 {
   SIPConnectionPrivate *priv = SIP_CONNECTION_GET_PRIVATE (obj);
   priv->sofia = sip_connection_sofia_new (obj);
+  priv->sofia_home = su_home_new(sizeof (su_home_t));
+  priv->auth_table = g_hash_table_new_full (g_direct_hash,
+                                            g_direct_equal,
+                                            NULL /* (GDestroyNotify) nua_handle_unref */,
+                                            g_free);
 }
 
 static void
@@ -236,18 +212,21 @@ sip_connection_set_property (GObject      *object,
     priv->password = g_value_dup_string (value);
     break;
   }
+  case PROP_TRANSPORT: {
+    g_free(priv->transport);
+    priv->transport = g_value_dup_string (value);
+    break;
+  }
   case PROP_PROXY: {
-    g_free(priv->proxy);
-    priv->proxy = priv_sip_strdup (g_value_get_string (value));
-    if (priv->sofia_nua) 
-      nua_set_params(priv->sofia_nua, NUTAG_PROXY(priv->proxy), TAG_END());
+    priv->proxy_url = priv_url_from_string_value (priv->sofia_home, value);
     break;
   }
   case PROP_REGISTRAR: {
-    g_free((gpointer)priv->registrar);
-    priv->registrar = priv_sip_strdup(g_value_get_string (value));
+    priv->registrar_url = priv_url_from_string_value (priv->sofia_home, value);
     if (priv->sofia_nua) 
-      nua_set_params(priv->sofia_nua, NUTAG_REGISTRAR(priv->registrar), TAG_END());
+      nua_set_params(priv->sofia_nua,
+                     NUTAG_REGISTRAR(priv->registrar_url),
+                     TAG_END());
     break;
   }
   case PROP_KEEPALIVE_MECHANISM: {
@@ -266,13 +245,6 @@ sip_connection_set_property (GObject      *object,
     }
     break;
   }
-  case PROP_HTTP_PROXY: {
-    g_free((gpointer)priv->http_proxy);
-    priv->http_proxy = g_value_dup_string (value);
-    if (priv->sofia_nua) 
-      nua_set_params(priv->sofia_nua, TPTAG_HTTP_CONNECT(priv->http_proxy), TAG_END());
-    break;
-  }
   case PROP_DISCOVER_BINDING: {
     priv->discover_binding = g_value_get_boolean (value);
     if (priv->sofia_nua)
@@ -291,6 +263,15 @@ sip_connection_set_property (GObject      *object,
     priv->stun_host = g_value_dup_string (value);
     break;
   }
+  case PROP_LOCAL_IP_ADDRESS: {
+    g_free (priv->local_ip_address);
+    priv->local_ip_address = g_value_dup_string (value);
+    break;
+  }
+  case PROP_LOCAL_PORT: {
+    priv->local_port = g_value_get_uint (value);
+    break;
+  }
   case PROP_EXTRA_AUTH_USER: {
     g_free((gpointer)priv->extra_auth_user);
     priv->extra_auth_user =  g_value_dup_string (value);
@@ -331,16 +312,20 @@ sip_connection_get_property (GObject      *object,
     g_value_set_string (value, priv->auth_user);
     break;
   }
+  case PROP_TRANSPORT: {
+    g_value_set_string (value, priv->transport);
+    break;
+  }
   case PROP_PASSWORD: {
     g_value_set_string (value, priv->password);
     break;
   }
   case PROP_PROXY: {
-    g_value_set_string (value, priv->proxy);
+    priv_value_set_url_as_string (value, priv->proxy_url);
     break;
   }
   case PROP_REGISTRAR: {
-    g_value_set_string (value, priv->registrar);
+    priv_value_set_url_as_string (value, priv->registrar_url);
     break;
   }
   case PROP_KEEPALIVE_MECHANISM: {
@@ -351,10 +336,6 @@ sip_connection_get_property (GObject      *object,
     g_value_set_int (value, priv->keepalive_interval);
     break;
   }
-  case PROP_HTTP_PROXY: {
-    g_value_set_string (value, priv->http_proxy);
-    break;
-  }
   case PROP_DISCOVER_BINDING: {
     g_value_set_boolean (value, priv->discover_binding);
     break;
@@ -370,6 +351,14 @@ sip_connection_get_property (GObject      *object,
     g_value_set_uint (value, priv->stun_port);
     break;
   }
+  case PROP_LOCAL_IP_ADDRESS: {
+    g_value_set_string (value, priv->local_ip_address);
+    break;
+  }
+  case PROP_LOCAL_PORT: {
+    g_value_set_uint (value, priv->local_port);
+    break;
+  }
   case PROP_SOFIA_ROOT: {
     g_return_if_fail (priv->sofia != NULL);
     g_value_set_pointer (value, priv->sofia->sofia_root);
@@ -423,8 +412,6 @@ sip_connection_class_init (SIPConnectionClass *sip_connection_class)
 
   g_type_class_add_private (sip_connection_class, sizeof (SIPConnectionPrivate));
 
-  object_class->constructor = sip_connection_constructor;
-
   object_class->dispose = sip_connection_dispose;
   object_class->finalize = sip_connection_finalize;
 
@@ -434,14 +421,20 @@ sip_connection_class_init (SIPConnectionClass *sip_connection_class)
   param_spec = g_param_spec_pointer("sofia-root",
                                     "Sofia root",
                                     "Event root from Sofia-SIP stack",
-                                    G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
+                                    G_PARAM_CONSTRUCT_ONLY |
+                                    G_PARAM_READWRITE |
+                                    G_PARAM_STATIC_NAME |
+                                    G_PARAM_STATIC_BLURB);
   INST_PROP(PROP_SOFIA_ROOT);
 
   param_spec = g_param_spec_string("address",
                                    "SIPConnection construction property",
                                    "Public SIP address",
                                    NULL, /*default value*/
-                                   G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
+                                   G_PARAM_CONSTRUCT_ONLY |
+                                   G_PARAM_READWRITE |
+                                   G_PARAM_STATIC_NAME |
+                                   G_PARAM_STATIC_BLURB);
   INST_PROP(PROP_ADDRESS);
 
   param_spec = g_param_spec_string("auth-user",
@@ -449,28 +442,45 @@ sip_connection_class_init (SIPConnectionClass *sip_connection_class)
                                    "Username to use when registering (if different "
                                    "than userinfo part of public SIP address)",
                                    NULL, /*default value*/
-                                   G_PARAM_READWRITE);
+                                   G_PARAM_READWRITE |
+                                   G_PARAM_STATIC_NAME |
+                                   G_PARAM_STATIC_BLURB);
   INST_PROP(PROP_AUTH_USER);
 
   param_spec = g_param_spec_string("password",
                                    "SIP account password",
                                    "Password for SIP registration",
                                    "", /*default value*/
-                                   G_PARAM_READWRITE);
+                                   G_PARAM_READWRITE |
+                                   G_PARAM_STATIC_NAME |
+                                   G_PARAM_STATIC_BLURB);
   INST_PROP(PROP_PASSWORD);
 
+  param_spec = g_param_spec_string("transport",
+                                   "Transport protocol",
+                                   "Preferred transport protocol [optional]",
+                                   NULL, /*default value*/
+                                   G_PARAM_READWRITE |
+                                   G_PARAM_STATIC_NAME |
+                                   G_PARAM_STATIC_BLURB);
+  INST_PROP(PROP_TRANSPORT);
+
   param_spec = g_param_spec_string("proxy",
                                    "Outbound proxy",
                                    "SIP URI for outbound proxy (e.g. 'sip:sipproxy.myprovider.com') [optional]",
                                    NULL, /*default value*/
-                                   G_PARAM_READWRITE);
+                                   G_PARAM_READWRITE |
+                                   G_PARAM_STATIC_NAME |
+                                   G_PARAM_STATIC_BLURB);
   INST_PROP(PROP_PROXY);
 
   param_spec = g_param_spec_string("registrar",
                                    "Registrar",
                                    "SIP URI for registrar (e.g. 'sip:sip.myprovider.com') [optional]",
                                    NULL, /*default value*/
-                                   G_PARAM_READWRITE);
+                                   G_PARAM_READWRITE |
+                                   G_PARAM_STATIC_NAME |
+                                   G_PARAM_STATIC_BLURB);
   INST_PROP(PROP_REGISTRAR);
 
   param_spec = g_param_spec_enum ("keepalive-mechanism",
@@ -478,49 +488,36 @@ sip_connection_class_init (SIPConnectionClass *sip_connection_class)
                                   "SIP registration keepalive mechanism",
                                   sip_connection_keepalive_mechanism_get_type (),
                                   SIP_CONNECTION_KEEPALIVE_AUTO,
-                                  G_PARAM_READWRITE);
+                                  G_PARAM_READWRITE |
+                                  G_PARAM_STATIC_NAME |
+                                  G_PARAM_STATIC_BLURB);
   INST_PROP(PROP_KEEPALIVE_MECHANISM);
 
   param_spec = g_param_spec_int("keepalive-interval", 
 				"Keepalive interval",
 				"Interval between keepalives in seconds (0 = disable, -1 = let stack decide.",
 				-1, G_MAXINT32, -1,
-				G_PARAM_READWRITE);
+                                G_PARAM_READWRITE |
+                                G_PARAM_STATIC_NAME |
+                                G_PARAM_STATIC_BLURB);
   INST_PROP(PROP_KEEPALIVE_INTERVAL);
 
-  param_spec = g_param_spec_string("http-proxy",
-                                   "HTTP proxy URI",
-                                   "Use HTTP-CONNECT to reach the SIP servers, empty to disable [optional]",
-                                   NULL, /*default value*/
-                                   G_PARAM_READWRITE);
-  INST_PROP(PROP_HTTP_PROXY);
-
   param_spec = g_param_spec_boolean("discover-binding",
                                     "Discover public contact",
                                     "Enable discovery of public IP address beyond NAT",
                                     TRUE, /*default value*/
-                                    G_PARAM_READWRITE);
+                                    G_PARAM_READWRITE |
+                                    G_PARAM_STATIC_NAME |
+                                    G_PARAM_STATIC_BLURB);
   INST_PROP(PROP_DISCOVER_BINDING);
 
-  param_spec = g_param_spec_string("extra-auth-user",
-                                   "Extra auth username",
-                                   "Username to use for extra authentication challenges",
-                                   NULL, /*default value*/
-                                   G_PARAM_READWRITE);
-  INST_PROP(PROP_EXTRA_AUTH_USER);
-
-  param_spec = g_param_spec_string("extra-auth-password",
-                                   "Extra auth password",
-                                   "Password to use for extra authentication challenges",
-                                   NULL, /*default value*/
-                                   G_PARAM_READWRITE);
-  INST_PROP(PROP_EXTRA_AUTH_PASSWORD);
-
   param_spec = g_param_spec_boolean("discover-stun", "Discover STUN server",
                                     "Enable discovery of STUN server host name "
                                     "using DNS SRV lookup",
                                     TRUE, /*default value*/
-                                    G_PARAM_READWRITE);
+                                    G_PARAM_READWRITE |
+                                    G_PARAM_STATIC_NAME |
+                                    G_PARAM_STATIC_BLURB);
   INST_PROP(PROP_DISCOVER_STUN);
 
   param_spec = g_param_spec_string("stun-server",
@@ -542,6 +539,42 @@ sip_connection_class_init (SIPConnectionClass *sip_connection_class)
                                   G_PARAM_STATIC_NAME |
                                   G_PARAM_STATIC_BLURB);
   INST_PROP(PROP_STUN_PORT);
+
+  param_spec = g_param_spec_string("local-ip-address",
+                                   "Local IP address",
+                                   "Local IP address to use [optional]",
+                                   NULL, /*default value*/
+                                   G_PARAM_READWRITE |
+                                   G_PARAM_STATIC_NAME |
+                                   G_PARAM_STATIC_BLURB);
+  INST_PROP(PROP_LOCAL_IP_ADDRESS);
+
+  param_spec = g_param_spec_uint ("local-port",
+                                  "Local port",
+                                  "Local port for SIP [optional]",
+                                  0, G_MAXUINT16, 0,
+                                  G_PARAM_READWRITE |
+                                  G_PARAM_STATIC_NAME |
+                                  G_PARAM_STATIC_BLURB);
+  INST_PROP(PROP_LOCAL_PORT);
+
+  param_spec = g_param_spec_string("extra-auth-user",
+                                   "Extra auth username",
+                                   "Username to use for extra authentication challenges",
+                                   NULL, /*default value*/
+                                   G_PARAM_READWRITE |
+                                   G_PARAM_STATIC_NAME |
+                                   G_PARAM_STATIC_BLURB);
+  INST_PROP(PROP_EXTRA_AUTH_USER);
+
+  param_spec = g_param_spec_string("extra-auth-password",
+                                   "Extra auth password",
+                                   "Password to use for extra authentication challenges",
+                                   NULL, /*default value*/
+                                   G_PARAM_READWRITE |
+                                   G_PARAM_STATIC_NAME |
+                                   G_PARAM_STATIC_BLURB);
+  INST_PROP(PROP_EXTRA_AUTH_PASSWORD);
 }
 
 static void
@@ -555,23 +588,21 @@ sip_connection_shut_down (TpBaseConnection *base)
   g_assert (SIP_IS_CONNECTION (self));
   priv = SIP_CONNECTION_GET_PRIVATE (self);
 
-  /* this shouldn't happen - shut_down is called when we change
-   * state to DISCONNECTED, and that should never happen unless we've
-   * started connecting
-   */
-  g_return_if_fail (priv->sofia_nua != NULL);
-
   /* We disposed of the REGISTER handle in the disconnected method */
   g_assert (priv->register_op == NULL);
 
   g_assert (priv->sofia != NULL);
 
   /* Detach the Sofia adapter and let it destroy the NUA handle and itself
-   * in the shutdown callback */
+   * in the shutdown callback. If there's no NUA stack, destroy it manually. */
   priv->sofia->conn = NULL;
-  priv->sofia = NULL;
 
-  nua_shutdown (priv->sofia_nua);
+  if (priv->sofia_nua != NULL)
+      nua_shutdown (priv->sofia_nua);
+  else
+      sip_connection_sofia_destroy (priv->sofia);
+
+  priv->sofia = NULL;
   priv->sofia_nua = NULL;
 
   tp_base_connection_finish_shutdown (base);
@@ -597,6 +628,9 @@ sip_connection_dispose (GObject *object)
   priv->media_factory = NULL;
   priv->text_factory = NULL;
 
+  /* may theoretically involve NUA handle unrefs */
+  g_hash_table_destroy (priv->auth_table);
+
   /* the base class is responsible for unreffing the self handle when we
    * disconnect */
   g_assert (base->status == TP_CONNECTION_STATUS_DISCONNECTED
@@ -619,7 +653,7 @@ sip_connection_finalize (GObject *obj)
 
   if (NULL != priv->sofia_resolver)
     {
-      g_debug ("destroying sofia resolver");
+      DEBUG("destroying sofia resolver");
       sres_resolver_destroy (priv->sofia_resolver);
       priv->sofia_resolver = NULL;
     }
@@ -629,15 +663,13 @@ sip_connection_finalize (GObject *obj)
   g_free (priv->address);
   g_free (priv->auth_user);
   g_free (priv->password);
-  g_free (priv->proxy);
-  g_free (priv->registrar);
-  g_free (priv->http_proxy);
+  g_free (priv->transport);
   g_free (priv->stun_host);
+  g_free (priv->local_ip_address);
   g_free (priv->extra_auth_user);
   g_free (priv->extra_auth_password);
 
   g_free (priv->registrar_realm);
-  g_free (priv->last_auth);
 
   G_OBJECT_CLASS (sip_connection_parent_class)->finalize (obj);
 }
@@ -658,6 +690,7 @@ sip_connection_start_connecting (TpBaseConnection *base,
   su_root_t *sofia_root;
   TpHandleRepoIface *contact_repo;
   const gchar *sip_address;
+  const url_t *local_url;
 
   g_message("%s: Connection %p ref-count=%u (obj)", G_STRFUNC, self,
       G_OBJECT(self)->ref_count);
@@ -680,8 +713,6 @@ sip_connection_start_connecting (TpBaseConnection *base,
       NULL, error);
   if (base->self_handle == 0)
     {
-      g_warning ("our requested SIP address '%s' is invalid, not connecting",
-          priv->address);
       return FALSE;
     }
 
@@ -690,8 +721,14 @@ sip_connection_start_connecting (TpBaseConnection *base,
   DEBUG("self_handle = %d, sip_address = %s", base->self_handle, sip_address);
 
   priv->account_url = url_make (priv->sofia_home, sip_address);
+  if (priv->account_url == NULL)
+    {
+      g_set_error (error, TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
+          "Failed to create the account URI");
+      return FALSE;
+    }
 
-  g_assert (priv->account_url != NULL);
+  local_url = sip_conn_get_local_url (self);
 
   /* step: create stack instance */
   priv->sofia_nua = nua_create (sofia_root,
@@ -699,7 +736,9 @@ sip_connection_start_connecting (TpBaseConnection *base,
       priv->sofia,
       SOATAG_AF(SOA_AF_IP4_IP6),
       SIPTAG_FROM_STR(sip_address),
-      NUTAG_URL("sip:*:*"),
+      NUTAG_URL(local_url),
+      TAG_IF(local_url && local_url->url_type == url_sips,
+             NUTAG_SIPS_URL(local_url)),
       NUTAG_M_USERNAME(priv->account_url->url_user),
       NUTAG_USER_AGENT("Telepathy-SofiaSIP/" TELEPATHY_SIP_VERSION),
       NUTAG_ENABLEMESSAGE(1),
@@ -716,25 +755,12 @@ sip_connection_start_connecting (TpBaseConnection *base,
       return FALSE;
     }
 
-  /* Take care about the proxy and contact URL */
-  if (priv->proxy != NULL)
-    {
-      nua_set_params (priv->sofia_nua,
-                      NUTAG_PROXY(priv->proxy),
-                      TAG_IF(g_ascii_strncasecmp(priv->proxy, "sips:", 5) == 0,
-                                                 NUTAG_SIPS_URL("sips:*")),
-                      TAG_NULL());
-    }
-  else if (g_ascii_strncasecmp(sip_address, "sips:", 5) == 0)
-    {
-      nua_set_params (priv->sofia_nua,
-                      NUTAG_SIPS_URL("sips:*"),
-                      TAG_NULL());
-    }
-
+  /* Set configuration-dependent tags */
+  sip_conn_update_proxy_and_transport (self);
   sip_conn_update_nua_outbound (self);
   sip_conn_update_nua_keepalive_interval (self);
   sip_conn_update_nua_contact_features (self);
+
   if (priv->stun_host != NULL)
     sip_conn_resolv_stun_server (self, priv->stun_host);
   else if (priv->discover_stun)
@@ -752,6 +778,13 @@ sip_connection_start_connecting (TpBaseConnection *base,
 
   priv->register_op = sip_conn_create_register_handle (self,
                                                        base->self_handle);
+  if (priv->register_op == NULL)
+    {
+      g_set_error (error, TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
+          "Unable to create registration handle for address %s", sip_address);
+      return FALSE;
+    }
+
   nua_register(priv->register_op, TAG_NULL());
 
   DEBUG("exit");
@@ -779,18 +812,9 @@ sip_connection_disconnected (TpBaseConnection *base)
   /* Dispose of the register use */
   if (priv->register_op != NULL)
     {
-      if (priv->register_succeeded)
-        {
-          DEBUG("unregistering");
-          nua_unregister (priv->register_op, TAG_NULL());
-          nua_handle_unref (priv->register_op);
-        }
-      else
-        {
-          DEBUG("don't let the incomplete registration linger any longer");
-          nua_handle_destroy (priv->register_op);
-        }
-
+      DEBUG("unregistering");
+      nua_unregister (priv->register_op, TAG_NULL());
+      nua_handle_unref (priv->register_op);
       priv->register_op = NULL;
     }
 }
@@ -855,11 +879,8 @@ sip_connection_request_handles (TpSvcConnection *iface,
   GArray *handles;
   GError *error = NULL;
   const gchar *client_name;
-  SIPConnectionPrivate *priv;
   TpHandleRepoIface *repo = tp_base_connection_get_handles (base, handle_type);
 
-  priv = SIP_CONNECTION_GET_PRIVATE (obj);
-
   DEBUG("enter");
 
   ERROR_IF_NOT_CONNECTED_ASYNC (base, context)
