@@ -36,7 +36,6 @@
 #include "media-factory.h"
 #include "sip-connection.h"
 #include "sip-media-session.h"
-#include "sip-media-stream.h"
 
 #include "telepathy-helpers.h"
 #include "sip-connection-helpers.h"
@@ -124,19 +123,12 @@ DEFINE_TP_STRUCT_TYPE(sip_session_handler_type,
 static void
 sip_media_channel_init (SIPMediaChannel *obj)
 {
-  SIPMediaChannelPrivate *priv = SIP_MEDIA_CHANNEL_GET_PRIVATE (obj);
-
-  DEBUG("enter");
-
   /* allocate any data required by the object here */
 
   /* initialise the properties mixin *before* GObject
    * sets the construct-time properties */
   tp_properties_mixin_init (G_OBJECT (obj),
       G_STRUCT_OFFSET (SIPMediaChannel, properties));
-
-  /* to keep the compiler happy */
-  priv = NULL;
 }
 
 static GObject *
@@ -189,8 +181,7 @@ static void sip_media_channel_set_property (GObject     *object,
 
 static void priv_create_session (SIPMediaChannel *channel,
                                  nua_handle_t *nh,
-                                 TpHandle peer,
-                                 gboolean remote_initiated);
+                                 TpHandle peer);
 static void priv_destroy_session(SIPMediaChannel *channel);
 static gboolean sip_media_channel_add_member (GObject *iface,
                                               TpHandle handle,
@@ -368,11 +359,9 @@ sip_media_channel_set_property (GObject     *object,
   SIPMediaChannelPrivate *priv = SIP_MEDIA_CHANNEL_GET_PRIVATE (chan);
 
   switch (property_id) {
-    case PROP_HANDLE:
-      /* this property is writable in the interface, but not actually
-       * meaningfully changable on this channel, so we do nothing */
-      break;
     case PROP_HANDLE_TYPE:
+    case PROP_HANDLE:
+    case PROP_CHANNEL_TYPE:
       /* this property is writable in the interface, but not actually
        * meaningfully changable on this channel, so we do nothing */
       break;
@@ -414,11 +403,11 @@ sip_media_channel_dispose (GObject *object)
   SIPMediaChannel *self = SIP_MEDIA_CHANNEL (object);
   SIPMediaChannelPrivate *priv = SIP_MEDIA_CHANNEL_GET_PRIVATE (self);
 
-  DEBUG("enter");
-
   if (priv->dispose_has_run)
     return;
 
+  DEBUG("enter");
+
   priv->dispose_has_run = TRUE;
 
   priv_destroy_session(self);
@@ -435,7 +424,7 @@ sip_media_channel_dispose (GObject *object)
   if (G_OBJECT_CLASS (sip_media_channel_parent_class)->dispose)
     G_OBJECT_CLASS (sip_media_channel_parent_class)->dispose (object);
 
-  DEBUG ("exit");
+  DEBUG("exit");
 }
 
 void
@@ -452,7 +441,7 @@ sip_media_channel_finalize (GObject *object)
 
   G_OBJECT_CLASS (sip_media_channel_parent_class)->finalize (object);
 
-  DEBUG ("exit");
+  DEBUG("exit");
 }
 
 /***********************************************************************
@@ -503,18 +492,13 @@ sip_media_channel_close (SIPMediaChannel *obj)
 void
 sip_media_channel_terminated (SIPMediaChannel *self)
 {
-  SIPMediaChannelPrivate *priv;
+  SIPMediaChannelPrivate *priv = SIP_MEDIA_CHANNEL_GET_PRIVATE (self);
 
   DEBUG("enter");
 
-  priv = SIP_MEDIA_CHANNEL_GET_PRIVATE (self);
-
   if (priv->session)
-    {
-      g_object_set (priv->session,
-                    "state", SIP_MEDIA_SESSION_STATE_ENDED,
-                    NULL);
-    }
+    sip_media_session_change_state (priv->session,
+                                    SIP_MEDIA_SESSION_STATE_ENDED);
 }
 
 /**
@@ -763,8 +747,6 @@ sip_media_channel_request_streams (TpSvcChannelTypeStreamedMedia *iface,
 
   DEBUG("enter");
 
-  g_assert (SIP_IS_MEDIA_CHANNEL (self));
-
   priv = SIP_MEDIA_CHANNEL_GET_PRIVATE (self);
 
   contact_repo = tp_base_connection_get_handles (
@@ -820,23 +802,20 @@ sip_media_channel_receive_invite (SIPMediaChannel *self,
                                   TpHandle handle)
 {
   SIPMediaChannelPrivate *priv = SIP_MEDIA_CHANNEL_GET_PRIVATE (self);
-  TpGroupMixin *mixin = TP_GROUP_MIXIN (self);
+  TpBaseConnection *conn = TP_BASE_CONNECTION (priv->conn);
   GObject *obj = G_OBJECT (self);
   TpHandleRepoIface *contact_repo;
   TpIntSet *member_set, *pending_set;
  
-  contact_repo = tp_base_connection_get_handles (
-        (TpBaseConnection *)(priv->conn), TP_HANDLE_TYPE_CONTACT);
+  contact_repo = tp_base_connection_get_handles (conn, TP_HANDLE_TYPE_CONTACT);
 
   if (priv->session == NULL)
     {
-      priv_create_session (self, nh, handle, TRUE);
-    
       /* note: start the local stream-engine; once the local 
-       *       candidate are ready, reply with nua_respond() 
-       *
-       *       with the tp-0.13 API, the streams need to be created
-       *       based on remote SDP (see sip_media_session_set_remote_media()) */
+       *       media are ready, reply with nua_respond() */
+      priv_create_session (self, nh, handle);
+      g_assert (priv->session != NULL);
+      sip_media_session_receive_invite (priv->session);
     }
   else
     g_warning ("session already exists");
@@ -854,7 +833,7 @@ sip_media_channel_receive_invite (SIPMediaChannel *self,
   tp_intset_add (member_set, handle);
 
   pending_set = tp_intset_new ();
-  tp_intset_add (pending_set, mixin->self_handle);
+  tp_intset_add (pending_set, conn->self_handle);
 
   tp_group_mixin_change_members (obj, "INVITE received",
                                  member_set,    /* add */
@@ -865,6 +844,11 @@ sip_media_channel_receive_invite (SIPMediaChannel *self,
 
   tp_intset_destroy (member_set);
   tp_intset_destroy (pending_set);
+
+  /* No adding more members to the incoming call, removing is OK */
+  tp_group_mixin_change_flags (obj,
+                               TP_CHANNEL_GROUP_FLAG_CAN_REMOVE,
+                               TP_CHANNEL_GROUP_FLAG_CAN_ADD);
 }
 
 /**
@@ -892,9 +876,17 @@ sip_media_channel_set_remote_media (SIPMediaChannel *chan,
 }
 
 void
-sip_media_channel_peer_response (SIPMediaChannel *self,
-                                 guint status,
-                                 const char* message)
+sip_media_channel_ready (SIPMediaChannel *self)
+{
+  SIPMediaChannelPrivate *priv = SIP_MEDIA_CHANNEL_GET_PRIVATE (self);
+  g_return_if_fail (priv->session != NULL);
+  sip_media_session_accept (priv->session);
+}
+
+void
+sip_media_channel_peer_error (SIPMediaChannel *self,
+                              guint status,
+                              const char* message)
 {
   SIPMediaChannelPrivate *priv = SIP_MEDIA_CHANNEL_GET_PRIVATE (self);
   TpIntSet *set;
@@ -905,19 +897,10 @@ sip_media_channel_peer_response (SIPMediaChannel *self,
 
   g_return_if_fail (priv->session != NULL);
 
-  g_return_if_fail (status >= 200);
-
-  if (status < 300)
-    {
-      /* The call has been successfully established,
-       * or a re-INVITE is complete */
-      sip_media_session_accept (priv->session);
-      return;
-    }
+  g_assert (status >= 300);
 
   switch (status)
     {
-    case 404:
     case 410:
     case 604:
       reason = TP_CHANNEL_GROUP_CHANGE_REASON_INVALID_CONTACT;
@@ -929,6 +912,7 @@ sip_media_channel_peer_response (SIPMediaChannel *self,
     case 408:
       reason = TP_CHANNEL_GROUP_CHANGE_REASON_NO_ANSWER;
       break;
+    case 404:
     case 480:
       reason = TP_CHANNEL_GROUP_CHANGE_REASON_OFFLINE;
       break;
@@ -945,6 +929,9 @@ sip_media_channel_peer_response (SIPMediaChannel *self,
 
   peer = sip_media_session_get_peer (priv->session);
 
+  if (message == NULL || !g_utf8_validate (message, -1, NULL))
+    message = "";
+
   set = tp_intset_new ();
   tp_intset_add (set, peer);
   tp_group_mixin_change_members ((GObject *)self, message,
@@ -953,22 +940,47 @@ sip_media_channel_peer_response (SIPMediaChannel *self,
 }
 
 void
-sip_media_channel_peer_cancel (SIPMediaChannel *self)
+sip_media_channel_peer_cancel (SIPMediaChannel *self,
+                               guint cause,
+                               const gchar *message)
 {
   SIPMediaChannelPrivate *priv = SIP_MEDIA_CHANNEL_GET_PRIVATE (self);
   TpGroupMixin *mixin = TP_GROUP_MIXIN (self);
   TpIntSet *set;
+  TpHandle actor = 0;
   TpHandle peer;
 
   g_return_if_fail (priv->session != NULL);
 
   peer = sip_media_session_get_peer (priv->session);
 
+  switch (cause)
+    {
+    case 200:
+    case 603:
+      /* The user must have acted on another branch of the forked call */
+      actor = mixin->self_handle;
+      break;
+    default:
+      actor = peer;
+    }
+
+  if (message == NULL || !g_utf8_validate (message, -1, NULL))
+    message = "Cancelled";
+
   set = tp_intset_new ();
   tp_intset_add (set, peer);
   tp_intset_add (set, mixin->self_handle);
-  tp_group_mixin_change_members ((GObject *) self, "Cancelled",
-      NULL, set, NULL, NULL, peer, TP_CHANNEL_GROUP_CHANGE_REASON_NONE);
+
+  tp_group_mixin_change_members ((GObject *) self,
+                                 message,
+                                 NULL, /* add */
+                                 set,  /* remove */
+                                 NULL,
+                                 NULL,
+                                 actor,
+                                 TP_CHANNEL_GROUP_CHANGE_REASON_NONE);
+
   tp_intset_destroy (set);
 }
 
@@ -977,31 +989,35 @@ sip_media_channel_peer_cancel (SIPMediaChannel *self)
  ***********************************************************************/
 
 static void priv_session_state_changed_cb (SIPMediaSession *session,
-					   GParamSpec *arg1,
+					   guint old_state,
+                                           guint state,
 					   SIPMediaChannel *channel)
 {
   TpGroupMixin *mixin = TP_GROUP_MIXIN (channel);
-  SIPMediaSessionState state;
   TpHandle peer;
   TpIntSet *set;
 
   DEBUG("enter");
 
-  g_object_get (session,
-                "state", &state,
-                "peer", &peer,
-                NULL);
+  peer = sip_media_session_get_peer (session);
 
   if (state == SIP_MEDIA_SESSION_STATE_ACTIVE) {
-    set = tp_intset_new ();
 
-    /* add the peer to the member list */
-    tp_intset_add (set, peer);
-
-    tp_group_mixin_change_members ((GObject *)channel,
-        "", set, NULL, NULL, NULL, 0, 0);
-
-    tp_intset_destroy (set);
+    if (old_state == SIP_MEDIA_SESSION_STATE_INVITE_SENT
+        || old_state == SIP_MEDIA_SESSION_STATE_RESPONSE_RECEIVED)
+      {
+        /* add the peer to the member list */
+        set = tp_intset_new ();
+        tp_intset_add (set, peer);
+        tp_group_mixin_change_members ((GObject *)channel,
+                                       "Call active",
+                                       set,     /* add */
+                                       NULL,    /* remove */
+                                       NULL,    /* local pending */
+                                       NULL,    /* remote pending */
+                                       0, 0);
+        tp_intset_destroy (set);
+      }
 
     /* update flags accordingly -- allow removal, deny adding and rescinding */
     tp_group_mixin_change_flags ((GObject *)channel,
@@ -1009,44 +1025,22 @@ static void priv_session_state_changed_cb (SIPMediaSession *session,
 				     TP_CHANNEL_GROUP_FLAG_CAN_ADD |
 				     TP_CHANNEL_GROUP_FLAG_CAN_RESCIND);
   }
-  else if (state == SIP_MEDIA_SESSION_STATE_ENDED) {
+  if (state == SIP_MEDIA_SESSION_STATE_ENDED) {
     set = tp_intset_new ();
 
     /* remove us and the peer from the member list */
     tp_intset_add (set, mixin->self_handle);
     tp_intset_add (set, peer);
     tp_group_mixin_change_members ((GObject *)channel,
-        "", NULL, set, NULL, NULL, 0, 0);
+        "Call ended", NULL, set, NULL, NULL, 0, 0);
 
     tp_intset_destroy (set);
 
-#if 0
-    /* update flags accordingly -- allow adding, deny removal */
-    tp_group_mixin_change_flags ((GObject *)channel,
-        TP_CHANNEL_GROUP_FLAG_CAN_ADD, TP_CHANNEL_GROUP_FLAG_CAN_REMOVE);
-#endif
-
     priv_destroy_session (channel);
     sip_media_channel_close (channel);
   }
 }
 
-static void priv_session_stream_added_cb (SIPMediaSession *session,
-					  SIPMediaStream  *stream,
-					  SIPMediaChannel *chan)
-{
-  guint id, handle, type;
-
-  DEBUG("enter");
-
-  /* emit StreamAdded */
-  handle = sip_media_session_get_peer (session);
-  g_object_get (stream, "id", &id, "media-type", &type, NULL);
-
-  tp_svc_channel_type_streamed_media_emit_stream_added (
-        (TpSvcChannelTypeStreamedMedia *)chan, id, handle, type);
-}
-
 /**
  * priv_create_session:
  *
@@ -1055,15 +1049,14 @@ static void priv_session_stream_added_cb (SIPMediaSession *session,
 static void
 priv_create_session (SIPMediaChannel *channel,
                      nua_handle_t *nh,
-                     TpHandle peer,
-                     gboolean remote_initiated)
+                     TpHandle peer)
 {
   SIPMediaChannelPrivate *priv;
   SIPMediaSession *session;
   TpBaseConnection *conn;
   TpHandleRepoIface *contact_repo;
   gchar *object_path;
-  const gchar *sid = NULL;
+  gchar *local_ip_address = NULL;
 
   DEBUG("enter");
 
@@ -1075,35 +1068,33 @@ priv_create_session (SIPMediaChannel *channel,
 
   object_path = g_strdup_printf ("%s/MediaSession%u", priv->object_path, peer);
 
-  /* allocate a hash-entry for the new media session */
-  sid = sip_media_factory_session_id_allocate (priv->factory);
-
   DEBUG("allocating session, peer=%u", peer);
 
   /* The channel manages references to the peer handle for the session */
   tp_handle_ref (contact_repo, peer);
 
+  g_object_get (priv->conn,
+                "local-ip-address", &local_ip_address,
+                NULL);
+
   session = g_object_new (SIP_TYPE_MEDIA_SESSION,
                           "media-channel", channel,
                           "object-path", object_path,
                           "nua-handle", nh,
-                          "session-id", sid,
                           "peer", peer,
+                          "local-ip-address", local_ip_address,
                           NULL);
 
-  g_signal_connect (session, "notify::state",
-                    (GCallback) priv_session_state_changed_cb, channel);
-  g_signal_connect (session, "stream-added",
-		    (GCallback) priv_session_stream_added_cb, channel);
+  g_free (local_ip_address);
+
+  g_signal_connect_object (session,
+                           "state-changed",
+                           G_CALLBACK(priv_session_state_changed_cb),
+                           channel,
+                           0);
 
   priv->session = session;
 
-  /* keep a list of media session ids */
-  sip_media_factory_session_id_register (priv->factory, sid, channel);
-
-  if (remote_initiated)
-    sip_media_session_receive_invite (session);
-
   tp_svc_channel_interface_media_signalling_emit_new_session_handler (
       (TpSvcChannelInterfaceMediaSignalling *)channel, object_path, "rtp");
 
@@ -1119,17 +1110,12 @@ priv_destroy_session(SIPMediaChannel *channel)
   SIPMediaSession *session;
   TpBaseConnection *conn;
   TpHandleRepoIface *contact_repo;
-  gchar *sid = NULL;
-
-  DEBUG("enter");
 
   session = priv->session;
   if (session == NULL)
     return;
 
-  g_object_get (session, "session-id", &sid, NULL);
-  sip_media_factory_session_id_unregister(priv->factory, sid);
-  g_free (sid);
+  DEBUG("enter");
 
   /* Release the peer handle */
   conn = (TpBaseConnection *)(priv->conn);
@@ -1139,6 +1125,8 @@ priv_destroy_session(SIPMediaChannel *channel)
 
   priv->session = NULL;
   g_object_unref (session);
+
+  DEBUG("exit");
 }
 
 #if 0
@@ -1198,14 +1186,13 @@ sip_media_channel_add_member (GObject *iface,
   /* case a: outgoing call (we are the initiator, a new handle added) */
   if (mixin->self_handle != handle)
     {
-      TpGroupMixin *mixin = TP_GROUP_MIXIN (self);
       TpIntSet *lset, *rset;
       nua_handle_t *nh;
 
       DEBUG("making outbound call - setting peer handle to %u", handle);
 
       nh = sip_conn_create_request_handle (priv->conn, handle);
-      priv_create_session (self, nh, handle, FALSE);
+      priv_create_session (self, nh, handle);
       nua_handle_unref (nh);
 
       /* make remote pending */
