@@ -41,6 +41,7 @@
 #include "sip-media-stream.h"
 #include "sip-connection-helpers.h"
 #include "telepathy-helpers.h"
+#include "signals-marshal.h"
 
 #define DEBUG_FLAG SIP_DEBUG_MEDIA
 #include "debug.h"
@@ -59,7 +60,7 @@ G_DEFINE_TYPE_WITH_CODE(SIPMediaSession,
 /* signal enum */
 enum
 {
-  SIG_STREAM_ADDED,
+  SIG_STATE_CHANGED,
   SIG_LAST_SIGNAL
 };
 
@@ -69,9 +70,8 @@ enum
   PROP_MEDIA_CHANNEL = 1,
   PROP_OBJECT_PATH,
   PROP_NUA_OP,
-  PROP_SESSION_ID,
   PROP_PEER,
-  PROP_STATE,
+  PROP_LOCAL_IP_ADDRESS,
   LAST_PROPERTY
 };
 
@@ -80,10 +80,11 @@ static guint signals[SIG_LAST_SIGNAL] = {0};
 #ifdef ENABLE_DEBUG
 
 /**
- * StreamEngine session states:
+ * Media session states:
  * - created, objects created, local cand/codec query ongoing
  * - invite-sent, an INVITE with local SDP sent, awaiting response
  * - invite-received, a remote INVITE received, response is pending
+ * - response-received, a 200 OK received, codec intersection is in progress
  * - active, codecs and candidate pairs have been negotiated (note,
  *   SteamEngine might still fail to verify connectivity and report
  *   an error)
@@ -96,6 +97,7 @@ static const char* session_states[] =
     "created",
     "invite-sent",
     "invite-received",
+    "response-received",
     "active",
     "reinvite-sent",
     "reinvite-received",
@@ -112,11 +114,12 @@ struct _SIPMediaSessionPrivate
   SIPMediaChannel *channel;             /** see gobj. prop. 'media-channel' */
   gchar *object_path;                   /** see gobj. prop. 'object-path' */
   nua_handle_t *nua_op;                 /** see gobj. prop. 'nua-handle' */
-  gchar *id;                            /** see gobj. prop. 'session-id' */
   TpHandle peer;                        /** see gobj. prop. 'peer' */
-  SIPMediaSessionState state;           /** see gobj. prop. 'state' */
+  gchar *local_ip_address;              /** see gobj. prop. 'local-ip-address' */
+  SIPMediaSessionState state;           /** session state */
   nua_saved_event_t saved_event[1];     /** Saved incoming request event */
   gint local_non_ready;                 /** number of streams with local information update pending */
+  guint remote_stream_count;            /** number of streams last seen in a remote offer */
   guint catcher_id;
   guint timer_id;
   su_home_t *home;                      /** Sofia memory home for remote SDP session structure */
@@ -146,8 +149,6 @@ sip_media_session_get_stream (SIPMediaSession *self,
                               guint stream_id,
                               GError **error);
 
-static void priv_session_state_changed (SIPMediaSession *session,
-                                        SIPMediaSessionState prev_state);
 static gboolean priv_catch_remote_nonupdate (gpointer data);
 static gboolean priv_timeout_session (gpointer data);
 static SIPMediaStream* priv_create_media_stream (SIPMediaSession *session,
@@ -222,14 +223,11 @@ static void sip_media_session_get_property (GObject    *object,
     case PROP_NUA_OP:
       g_value_set_pointer (value, priv->nua_op);
       break;
-    case PROP_SESSION_ID:
-      g_value_set_string (value, priv->id);
-      break;
     case PROP_PEER:
       g_value_set_uint (value, priv->peer);
       break;
-    case PROP_STATE:
-      g_value_set_uint (value, priv->state);
+    case PROP_LOCAL_IP_ADDRESS:
+      g_value_set_string (value, priv->local_ip_address);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -247,10 +245,10 @@ static void sip_media_session_set_property (GObject      *object,
 
   switch (property_id) {
     case PROP_MEDIA_CHANNEL:
-      priv->channel = SIP_MEDIA_CHANNEL (g_value_dup_object (value));
+      priv->channel = SIP_MEDIA_CHANNEL (g_value_get_object (value));
       break;
     case PROP_OBJECT_PATH:
-      g_free (priv->object_path);
+      g_assert (priv->object_path == NULL);
       priv->object_path = g_value_dup_string (value);
       break;
     case PROP_NUA_OP:
@@ -260,15 +258,12 @@ static void sip_media_session_set_property (GObject      *object,
       priv->nua_op = g_value_get_pointer (value);
       nua_handle_ref (priv->nua_op);
       break;
-    case PROP_SESSION_ID:
-      g_free (priv->id);
-      priv->id = g_value_dup_string (value);
-      break;
     case PROP_PEER:
       priv->peer = g_value_get_uint (value);
       break;
-    case PROP_STATE:
-      priv_session_state_changed (session, g_value_get_uint (value));
+    case PROP_LOCAL_IP_ADDRESS:
+      g_assert (priv->local_ip_address == NULL);
+      priv->local_ip_address = g_value_dup_string (value);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -297,7 +292,7 @@ sip_media_session_class_init (SIPMediaSessionClass *sip_media_session_class)
 
   param_spec = g_param_spec_object ("media-channel", "SIPMediaChannel object",
                                     "SIP media channel object that owns this "
-                                    "media session object.",
+                                    "media session object (not reference counted).",
                                     SIP_TYPE_MEDIA_CHANNEL,
                                     G_PARAM_CONSTRUCT_ONLY |
                                     G_PARAM_READWRITE |
@@ -324,16 +319,6 @@ sip_media_session_class_init (SIPMediaSessionClass *sip_media_session_class)
                                     G_PARAM_STATIC_BLURB);
   g_object_class_install_property (object_class, PROP_NUA_OP, param_spec);
 
-  param_spec = g_param_spec_string ("session-id", "Session ID",
-                                    "A unique session identifier used "
-                                    "throughout all communication.",
-				    NULL,
-                                    G_PARAM_CONSTRUCT_ONLY |
-                                    G_PARAM_READWRITE |
-                                    G_PARAM_STATIC_NAME |
-                                    G_PARAM_STATIC_BLURB);
-  g_object_class_install_property (object_class, PROP_SESSION_ID, param_spec);
-
   param_spec = g_param_spec_uint ("peer", "Session peer",
                                   "The TpHandle representing the contact "
                                   "with whom this session communicates.",
@@ -344,22 +329,24 @@ sip_media_session_class_init (SIPMediaSessionClass *sip_media_session_class)
                                   G_PARAM_STATIC_BLURB);
   g_object_class_install_property (object_class, PROP_PEER, param_spec);
 
-  param_spec = g_param_spec_uint ("state", "Session state",
-                                  "The current state that the session is in.",
-                                  0, G_MAXUINT32, 0,
-                                  G_PARAM_READWRITE |
-                                  G_PARAM_STATIC_NAME |
-                                  G_PARAM_STATIC_BLURB);
-  g_object_class_install_property (object_class, PROP_STATE, param_spec);
+  param_spec = g_param_spec_string ("local-ip-address", "Local IP address",
+                                    "The local IP address preferred for "
+                                    "media streams",
+                                    NULL,
+                                    G_PARAM_CONSTRUCT_ONLY |
+                                    G_PARAM_READWRITE |
+                                    G_PARAM_STATIC_NAME |
+                                    G_PARAM_STATIC_BLURB);
+  g_object_class_install_property (object_class, PROP_LOCAL_IP_ADDRESS, param_spec);
 
-  signals[SIG_STREAM_ADDED] =
-    g_signal_new ("stream-added",
+  signals[SIG_STATE_CHANGED] =
+    g_signal_new ("state-changed",
                   G_OBJECT_CLASS_TYPE (sip_media_session_class),
                   G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
                   0,
                   NULL, NULL,
-                  g_cclosure_marshal_VOID__OBJECT,
-                  G_TYPE_NONE, 1, G_TYPE_OBJECT);
+                  _tpsip_marshal_VOID__UINT_UINT,
+                  G_TYPE_NONE, 2, G_TYPE_UINT, G_TYPE_UINT);
 }
 
 static void
@@ -371,6 +358,8 @@ sip_media_session_dispose (GObject *object)
   if (priv->dispose_has_run)
     return;
 
+  DEBUG("enter");
+
   priv->dispose_has_run = TRUE;
 
   if (priv->catcher_id)
@@ -379,31 +368,24 @@ sip_media_session_dispose (GObject *object)
   if (priv->timer_id)
     g_source_remove (priv->timer_id);
 
-  if (priv->channel)
-    g_object_unref (priv->channel);
-
   if (G_OBJECT_CLASS (sip_media_session_parent_class)->dispose)
     G_OBJECT_CLASS (sip_media_session_parent_class)->dispose (object);
 
-  DEBUG ("exit");
+  DEBUG("exit");
 }
 
-void
+static void
 sip_media_session_finalize (GObject *object)
 {
   SIPMediaSession *self = SIP_MEDIA_SESSION (object);
   SIPMediaSessionPrivate *priv = SIP_MEDIA_SESSION_GET_PRIVATE (self);
   guint i;
 
-  DEBUG ("enter");
-
-  /* free any data held directly by the object here */
-
-  G_OBJECT_CLASS (sip_media_session_parent_class)->finalize (object);
-
   /* terminating the session should have discarded the NUA handle */
   g_assert (priv->nua_op == NULL);
 
+  /* free any data held directly by the object here */
+
   for (i = 0; i < priv->streams->len; i++) {
     SIPMediaStream *stream = g_ptr_array_index (priv->streams, i);
     if (stream != NULL)
@@ -421,7 +403,12 @@ sip_media_session_finalize (GObject *object)
   if (priv->backup_home != NULL)
     su_home_unref (priv->backup_home);
 
-  DEBUG ("exit");
+  g_free (priv->local_ip_address);
+  g_free (priv->object_path);
+
+  G_OBJECT_CLASS (sip_media_session_parent_class)->finalize (object);
+
+  DEBUG("exit");
 }
 
 
@@ -533,41 +520,38 @@ priv_close_all_streams (SIPMediaSession *session)
 }
 
 static void
-priv_clear_streams_pending_send (SIPMediaSession *session)
+priv_release_streams_pending_send (SIPMediaSession *session)
 {
   SIPMediaSessionPrivate *priv = SIP_MEDIA_SESSION_GET_PRIVATE (session);
   SIPMediaStream *stream;
   guint i;
 
-  /* Clear the local pending send flags, enabling sending */
+  /* Clear the local pending send flags where applicable */
   for (i = 0; i < priv->streams->len; i++)
     {
       stream = g_ptr_array_index(priv->streams, i);
       if (stream != NULL)
-        {
-          guint direction = TP_MEDIA_STREAM_DIRECTION_BIDIRECTIONAL;
-          g_object_get (stream, "direction", &direction, NULL);
-          if (direction & TP_MEDIA_STREAM_DIRECTION_SEND)
-            sip_media_stream_set_direction (stream, direction, 0);
-        }
+        sip_media_stream_release_pending_send (stream);
     }
 }
 
-static void
-priv_session_state_changed (SIPMediaSession *session,
-                            SIPMediaSessionState new_state)
+void
+sip_media_session_change_state (SIPMediaSession *session,
+                                SIPMediaSessionState new_state)
 {
   SIPMediaSessionPrivate *priv = SIP_MEDIA_SESSION_GET_PRIVATE (session);
+  guint old_state;
 
   if (priv->state == new_state)
     return;
 
-  SESSION_DEBUG(session, "state changed from %s to %s",
-                session_states[priv->state],
-                session_states[new_state]);
-
+  old_state = priv->state;
   priv->state = new_state;
 
+  SESSION_DEBUG(session, "state change: %s -> %s",
+                session_states[old_state],
+                session_states[new_state]);
+
   switch (new_state)
     {
     case SIP_MEDIA_SESSION_STATE_CREATED:
@@ -595,22 +579,25 @@ priv_session_state_changed (SIPMediaSession *session,
       priv->timer_id =
         g_timeout_add (DEFAULT_SESSION_TIMEOUT, priv_timeout_session, session);
       break;
+    case SIP_MEDIA_SESSION_STATE_RESPONSE_RECEIVED:
+      break;
     case SIP_MEDIA_SESSION_STATE_ACTIVE:
       if (priv->timer_id)
         {
 	  g_source_remove (priv->timer_id);
 	  priv->timer_id = 0;
         }
-      priv_clear_streams_pending_send (session);
-      if (priv->pending_offer)
-        {
-          priv_session_invite (session, TRUE);
-        }
+      priv_release_streams_pending_send (session);
       break;
 
     /* Don't add default because we want to be warned by the compiler
      * about unhandled states */
     }
+
+  g_signal_emit (session, signals[SIG_STATE_CHANGED], 0, old_state, new_state);
+
+  if (new_state == SIP_MEDIA_SESSION_STATE_ACTIVE && priv->pending_offer)
+    priv_session_invite (session, TRUE);
 }
 
 #ifdef ENABLE_DEBUG
@@ -662,6 +649,9 @@ static gboolean priv_timeout_session (gpointer data)
 {
   SIPMediaSession *session = data;
   SIPMediaSessionPrivate *priv;
+  TpChannelGroupChangeReason reason;
+  gboolean change = FALSE;
+  TpHandle actor;
 
   DEBUG("session timed out");
 
@@ -669,12 +659,23 @@ static gboolean priv_timeout_session (gpointer data)
 
   if (priv->state == SIP_MEDIA_SESSION_STATE_INVITE_SENT)
     {
-      TpIntSet *set;
-      set = tp_intset_new ();
+      reason = TP_CHANNEL_GROUP_CHANGE_REASON_NO_ANSWER;
+      actor = 0;
+      change = TRUE;
+    }
+  else if (priv->state == SIP_MEDIA_SESSION_STATE_INVITE_RECEIVED)
+    {
+      reason = TP_CHANNEL_GROUP_CHANGE_REASON_NONE;
+      actor = priv->peer;
+      change = TRUE;
+    }
+
+  if (change)
+    {
+      TpIntSet *set = tp_intset_new ();
       tp_intset_add (set, priv->peer);
       tp_group_mixin_change_members (G_OBJECT (priv->channel), "Timed out",
-                                     NULL, set, NULL, NULL, 0,
-                                     TP_CHANNEL_GROUP_CHANGE_REASON_NO_ANSWER);
+                                     NULL, set, NULL, NULL, actor, reason);
       tp_intset_destroy (set);
     }
 
@@ -692,6 +693,7 @@ void sip_media_session_terminate (SIPMediaSession *session)
   if (priv->state == SIP_MEDIA_SESSION_STATE_ENDED)
     return;
 
+  /* XXX: taken care of by the state change? */
   priv_close_all_streams (session);
 
   if (priv->nua_op != NULL)
@@ -701,6 +703,7 @@ void sip_media_session_terminate (SIPMediaSession *session)
       switch (priv->state)
         {
         case SIP_MEDIA_SESSION_STATE_ACTIVE:
+        case SIP_MEDIA_SESSION_STATE_RESPONSE_RECEIVED:
         case SIP_MEDIA_SESSION_STATE_REINVITE_SENT:
           DEBUG("sending BYE");
           nua_bye (priv->nua_op, TAG_END());
@@ -714,17 +717,14 @@ void sip_media_session_terminate (SIPMediaSession *session)
           nua_respond (priv->nua_op, 480, "Terminated", TAG_END());
           break;
         case SIP_MEDIA_SESSION_STATE_REINVITE_RECEIVED:
-          DEBUG("sending the 480 response to an incoming re-INVITE");
-          {
-            msg_t *msg;
-
-            msg = (priv->saved_event[0])
-                        ? nua_saved_event_request (priv->saved_event) : NULL;
-                
-            nua_respond (priv->nua_op, 480, "Terminated",
-                         TAG_IF(msg, NUTAG_WITH(msg)),
-                         TAG_END());
-          }
+          if (priv->saved_event[0])
+            {
+              DEBUG("sending the 480 response to an incoming re-INVITE");
+              nua_respond (priv->nua_op, 480, "Terminated",
+                           NUTAG_WITH(nua_saved_event_request (priv->saved_event)),
+                           TAG_END());
+              nua_destroy_event (priv->saved_event);
+            }
           DEBUG("sending BYE to terminate the call itself");
           nua_bye (priv->nua_op, TAG_END());
           break;
@@ -733,7 +733,7 @@ void sip_media_session_terminate (SIPMediaSession *session)
         }
     }
 
-  g_object_set (session, "state", SIP_MEDIA_SESSION_STATE_ENDED, NULL);
+  sip_media_session_change_state (session, SIP_MEDIA_SESSION_STATE_ENDED);
 }
 
 gboolean
@@ -751,6 +751,26 @@ sip_media_session_set_remote_media (SIPMediaSession *session,
       priv->catcher_id = 0;
     }
 
+  if (priv->state == SIP_MEDIA_SESSION_STATE_INVITE_SENT
+      || priv->state == SIP_MEDIA_SESSION_STATE_REINVITE_SENT)
+    {
+      sip_media_session_change_state (
+                session,
+                SIP_MEDIA_SESSION_STATE_RESPONSE_RECEIVED);
+    }
+  else
+    {
+      /* Remember the m= line count in the remote offer,
+       * to match it with exactly this number of answer lines */
+      sdp_media_t *media;
+      guint count = 0;
+
+      for (media = sdp->sdp_media; media != NULL; media = media->m_next)
+        ++count;
+
+      priv->remote_stream_count = count;
+    }
+
   /* Handle session non-updates */
   if (!sdp_session_cmp (priv->remote_sdp, sdp))
     {
@@ -926,28 +946,30 @@ sip_media_session_request_stream_direction (SIPMediaSession *self,
                                             guint direction,
                                             GError **error)
 {
+  SIPMediaSessionPrivate *priv = SIP_MEDIA_SESSION_GET_PRIVATE (self);
   SIPMediaStream *stream;
-  guint old_direction = TP_MEDIA_STREAM_DIRECTION_BIDIRECTIONAL;
-  guint pending_send_flags = 0;
 
   stream = sip_media_session_get_stream (self, stream_id, error);
   if (stream == NULL)
-    return FALSE;
+    {
+      g_set_error (error, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+                   "stream %u does not exist", stream_id);
+      return FALSE;
+    }
 
-  g_object_get (stream,
-                "direction", &old_direction,
-                "pending-send-flags", &pending_send_flags,
-                NULL);
+  if (priv->state == SIP_MEDIA_SESSION_STATE_INVITE_RECEIVED
+      || priv->state == SIP_MEDIA_SESSION_STATE_REINVITE_RECEIVED)
+    {
+      /* While processing a session offer, we can only mask out direction
+       * requested by the remote peer */
+      guint old_direction = TP_MEDIA_STREAM_DIRECTION_BIDIRECTIONAL;
+      g_object_get (stream,
+                    "direction", &old_direction,
+                    NULL);
+      direction &= old_direction;
+    }
 
-  SESSION_DEBUG(self, "stream %u direction change requested: %u -> %u", stream_id, old_direction, direction);
-
-  /* Set pending send flag if we're going to start sending */
-  if (direction & ~old_direction & TP_MEDIA_STREAM_DIRECTION_SEND)
-    pending_send_flags |= TP_MEDIA_STREAM_PENDING_REMOTE_SEND;
-
-  sip_media_stream_set_direction (stream, direction, pending_send_flags);
-
-  if (direction != old_direction)
+  if (sip_media_stream_set_direction (stream, direction, FALSE))
     priv_local_media_changed (self);
 
   return TRUE;
@@ -1002,7 +1024,7 @@ sip_media_session_receive_invite (SIPMediaSession *self)
 
   nua_respond (priv->nua_op, SIP_180_RINGING, TAG_END());
 
-  g_object_set (self, "state", SIP_MEDIA_SESSION_STATE_INVITE_RECEIVED, NULL);
+  sip_media_session_change_state (self, SIP_MEDIA_SESSION_STATE_INVITE_RECEIVED);
 }
 
 void
@@ -1010,18 +1032,12 @@ sip_media_session_receive_reinvite (SIPMediaSession *self)
 {
   SIPMediaSessionPrivate *priv = SIP_MEDIA_SESSION_GET_PRIVATE (self);
 
-  /* Note: the Sofia-SIP stack is supposed to weed out wrongly sequenced
-   * INVITE requests with session offers; here we may have a case
-   * when the processing of the response did not complete before we
-   * return back to active */
-
   g_return_if_fail (priv->state == SIP_MEDIA_SESSION_STATE_ACTIVE
-                    || priv->state == SIP_MEDIA_SESSION_STATE_INVITE_SENT
-                    || priv->state == SIP_MEDIA_SESSION_STATE_REINVITE_SENT);
+                    || priv->state == SIP_MEDIA_SESSION_STATE_RESPONSE_RECEIVED);
 
   priv_save_event (self);
 
-  g_object_set (self, "state", SIP_MEDIA_SESSION_STATE_REINVITE_RECEIVED, NULL);
+  sip_media_session_change_state (self, SIP_MEDIA_SESSION_STATE_REINVITE_RECEIVED);
 }
 
 void
@@ -1137,11 +1153,35 @@ sip_media_session_stop_telephony_event  (SIPMediaSession *self,
   return TRUE;
 }
 
-/***********************************************************************
- * Helper functions follow (not based on generated templates)
- ***********************************************************************/
+gint
+sip_media_session_rate_native_transport (SIPMediaSession *session,
+                                         const GValue *transport)
+{
+  SIPMediaSessionPrivate *priv = SIP_MEDIA_SESSION_GET_PRIVATE (session);
+  gint result = 0;
+  gchar *address = NULL;
+  guint proto = TP_MEDIA_STREAM_BASE_PROTO_UDP;
 
-static void priv_session_media_state (SIPMediaSession *session, gboolean playing)
+  dbus_g_type_struct_get (transport,
+                          1, &address,
+                          3, &proto,
+                          G_MAXUINT);
+
+  g_assert (address != NULL);
+
+  if (proto != TP_MEDIA_STREAM_BASE_PROTO_UDP)
+    result = -1;
+  /* XXX: this will not work properly when IPv6 support comes */
+  else if (priv->local_ip_address != NULL
+      && strcmp (address, priv->local_ip_address) == 0)
+    result = 1;
+
+  g_free (address);
+
+  return result;
+}
+
+static void priv_session_set_streams_playing (SIPMediaSession *session, gboolean playing)
 {
   SIPMediaSessionPrivate *priv = SIP_MEDIA_SESSION_GET_PRIVATE (session);
   SIPMediaStream *stream;
@@ -1163,14 +1203,22 @@ priv_local_media_changed (SIPMediaSession *session)
   switch (priv->state)
     {
     case SIP_MEDIA_SESSION_STATE_CREATED:
+      /* If all streams are ready, send an offer now */
+      priv_request_response_step (session);
+      break;
     case SIP_MEDIA_SESSION_STATE_INVITE_RECEIVED:
     case SIP_MEDIA_SESSION_STATE_REINVITE_RECEIVED:
-      /* The changes will be sent when all streams are ready;
-       * check if now's the time */
-      priv_request_response_step (session);
+      /* The changes to existing streams will be included in the
+       * eventual answer (FIXME: implement postponed direction changes,
+       * which are applied after the remote offer has been processed).
+       * Check, however, if there are new streams not present in the
+       * remote offer, that will need another offer-answer round */
+      if (priv->remote_stream_count < priv->streams->len)
+        priv->pending_offer = TRUE;
       break;
     case SIP_MEDIA_SESSION_STATE_INVITE_SENT:
     case SIP_MEDIA_SESSION_STATE_REINVITE_SENT:
+    case SIP_MEDIA_SESSION_STATE_RESPONSE_RECEIVED:
       /* Cannot send another offer right now */
       priv->pending_offer = TRUE;
       break;
@@ -1244,14 +1292,25 @@ priv_update_remote_media (SIPMediaSession *session, gboolean authoritative)
         }
 
       /* There have been problems with the stream update, kill the stream */
-      /* XXX: fast and furious, not tested */
       sip_media_stream_close (stream);
     }
-
   g_assert(media == NULL);
   g_assert(i <= priv->streams->len);
-  if (i < priv->streams->len)
+  g_assert(!authoritative || i == priv->remote_stream_count);
+
+  if (i < priv->streams->len && !priv->pending_offer)
     {
+      /*
+       * It's not defined what we should do if there are previously offered
+       * streams not accounted in the remote SDP, in violation of RFC 3264.
+       * Closing them off serves resource preservation and gives better
+       * clue to the client as to the real state of the session.
+       * Note that this situation is masked if any local media updates
+       * have been requested and are pending until the remote session answer
+       * has been received and applied. In such a case, we'll issue a new offer
+       * at the closest available time, with the "overhanging" stream entries
+       * intact.
+       */
       do
         {
           SIPMediaStream *stream;
@@ -1274,7 +1333,6 @@ static void
 priv_session_rollback (SIPMediaSession *session)
 {
   SIPMediaSessionPrivate *priv = SIP_MEDIA_SESSION_GET_PRIVATE (session);
-  msg_t *msg;
 
   DEBUG("enter");
 
@@ -1300,31 +1358,44 @@ priv_session_rollback (SIPMediaSession *session)
 
   priv_update_remote_media (session, FALSE);
 
-  msg = (priv->saved_event[0])
-        ? nua_saved_event_request (priv->saved_event) : NULL;
-
-  nua_respond (priv->nua_op, 488, sip_488_Not_acceptable,
-               TAG_IF(msg, NUTAG_WITH(msg)),
-               TAG_END());
-
   if (priv->saved_event[0])
-    nua_destroy_event (priv->saved_event);
+    {
+      nua_respond (priv->nua_op, SIP_488_NOT_ACCEPTABLE,
+                   NUTAG_WITH(nua_saved_event_request (priv->saved_event)),
+                   TAG_END());
+      nua_destroy_event (priv->saved_event);
+    }
+  else
+    {
+      nua_respond (priv->nua_op, SIP_488_NOT_ACCEPTABLE,
+                   TAG_END());
+    }
 
-  g_object_set (session, "state", SIP_MEDIA_SESSION_STATE_ACTIVE, NULL);
+  sip_media_session_change_state (session, SIP_MEDIA_SESSION_STATE_ACTIVE);
 }
 
 static gboolean
-priv_session_local_sdp (SIPMediaSession *session, GString *user_sdp)
+priv_session_local_sdp (SIPMediaSession *session,
+                        GString *user_sdp,
+                        gboolean authoritative)
 {
   SIPMediaSessionPrivate *priv = SIP_MEDIA_SESSION_GET_PRIVATE (session);
   gboolean has_supported_media = FALSE;
+  guint len;
   guint i;
 
   g_return_val_if_fail (priv->local_non_ready == 0, FALSE);
 
+  len = priv->streams->len;
+  if (!authoritative && len > priv->remote_stream_count)
+    {
+      len = priv->remote_stream_count;
+      DEBUG("clamped response to %u streams seen in the offer", len);
+    }
+
   g_string_append (user_sdp, "v=0\r\n");
 
-  for (i = 0; i < priv->streams->len; i++)
+  for (i = 0; i < len; i++)
     {
       SIPMediaStream *stream = g_ptr_array_index (priv->streams, i);
       if (stream)
@@ -1356,7 +1427,7 @@ priv_session_invite (SIPMediaSession *session, gboolean reinvite)
 
   user_sdp = g_string_new (NULL);
 
-  if (priv_session_local_sdp (session, user_sdp))
+  if (priv_session_local_sdp (session, user_sdp, TRUE))
     {
       nua_invite (priv->nua_op,
                   SOATAG_USER_SDP_STR(user_sdp->str),
@@ -1365,10 +1436,10 @@ priv_session_invite (SIPMediaSession *session, gboolean reinvite)
                   NUTAG_AUTOANSWER(0),
                   TAG_END());
       priv->pending_offer = FALSE;
-      g_object_set (session,
-                    "state", reinvite? SIP_MEDIA_SESSION_STATE_REINVITE_SENT
-                                     : SIP_MEDIA_SESSION_STATE_INVITE_SENT,
-                    NULL);
+      sip_media_session_change_state (
+                session,
+                reinvite? SIP_MEDIA_SESSION_STATE_REINVITE_SENT
+                        : SIP_MEDIA_SESSION_STATE_INVITE_SENT);
     }
   else
     g_warning ("cannot send a valid SDP offer, are there no streams?");
@@ -1386,7 +1457,7 @@ priv_session_respond (SIPMediaSession *session)
 
   user_sdp = g_string_new (NULL);
 
-  if (priv_session_local_sdp (session, user_sdp))
+  if (priv_session_local_sdp (session, user_sdp, FALSE))
     {
       msg_t *msg;
 
@@ -1404,7 +1475,7 @@ priv_session_respond (SIPMediaSession *session)
       if (priv->saved_event[0])
         nua_destroy_event (priv->saved_event);
 
-      g_object_set (session, "state", SIP_MEDIA_SESSION_STATE_ACTIVE, NULL);
+      sip_media_session_change_state (session, SIP_MEDIA_SESSION_STATE_ACTIVE);
     }
   else
     {
@@ -1447,62 +1518,49 @@ priv_request_response_step (SIPMediaSession *session)
 {
   SIPMediaSessionPrivate *priv = SIP_MEDIA_SESSION_GET_PRIVATE (session);
 
-  DEBUG ("enter, local non ready %d", priv->local_non_ready);
+  if (priv->local_non_ready != 0)
+    {
+      DEBUG("there are local streams not ready, postponed");
+      return;
+    }
 
   switch (priv->state)
     {
     case SIP_MEDIA_SESSION_STATE_CREATED:
-      if (priv->local_non_ready == 0)
-        {
-          /* note:  we need to be prepared to receive media right after the
-           *       offer is sent, so we must set state to playing */
-          priv_session_media_state (session, TRUE);
-
-          priv_session_invite (session, FALSE);
-        }
+      /* note: we need to be prepared to receive media right after the
+       *       offer is sent, so we must set the streams to playing */
+      priv_session_set_streams_playing (session, TRUE);
+      priv_session_invite (session, FALSE);
       break;
-    case SIP_MEDIA_SESSION_STATE_INVITE_SENT:
-      if (!priv->accepted)
-        break;
-      /* if accepted, fall through to the next case */
-    case SIP_MEDIA_SESSION_STATE_REINVITE_SENT:
-      if (!priv_is_codec_intersect_pending (session))
-        {
-          g_assert (priv->local_non_ready == 0);
-          g_object_set (session,
-                        "state", SIP_MEDIA_SESSION_STATE_ACTIVE,
-                        NULL);
-        }
+    case SIP_MEDIA_SESSION_STATE_RESPONSE_RECEIVED:
+      if (priv->accepted
+          && !priv_is_codec_intersect_pending (session))
+        sip_media_session_change_state (session,
+                                        SIP_MEDIA_SESSION_STATE_ACTIVE);
       break;
     case SIP_MEDIA_SESSION_STATE_INVITE_RECEIVED:
       /* TODO: if the call has not yet been accepted locally
        * and the remote endpoint supports 100rel, send them
        * an early session answer in a reliable 183 response */
       if (priv->accepted
-          && priv->local_non_ready == 0
           && !priv_is_codec_intersect_pending (session))
         {
           priv_session_respond (session);
 
-          /* note: we have accepted the call, set state to playing */ 
-          priv_session_media_state (session, TRUE);
+          /* note: we have accepted the call, set the streams to playing */
+          priv_session_set_streams_playing (session, TRUE);
         }
       break;
     case SIP_MEDIA_SESSION_STATE_REINVITE_RECEIVED:
-      if (priv->local_non_ready == 0
-          && !priv_is_codec_intersect_pending (session))
-        {
-          priv_session_respond (session);
-        }
+      if (!priv_is_codec_intersect_pending (session))
+        priv_session_respond (session);
       break;
     case SIP_MEDIA_SESSION_STATE_ACTIVE:
-      if (priv->pending_offer && priv->local_non_ready == 0)
-        {
-          priv_session_invite (session, TRUE);
-        }
+      if (priv->pending_offer)
+        priv_session_invite (session, TRUE);
       break;
     default:
-      g_assert_not_reached ();
+      SESSION_DEBUG(session, "no action taken in the current state");
     }
 }
 
@@ -1520,7 +1578,7 @@ priv_stream_close_cb (SIPMediaStream *stream,
   id = sip_media_stream_get_id (stream);
   g_return_if_fail (g_ptr_array_index(priv->streams, id) == stream);
 
-  if (!sip_media_stream_is_ready (stream))
+  if (!sip_media_stream_is_local_ready (stream))
     {
       g_assert (priv->local_non_ready > 0);
       --priv->local_non_ready;
@@ -1530,6 +1588,8 @@ priv_stream_close_cb (SIPMediaStream *stream,
   g_object_unref (stream);
 
   g_ptr_array_index(priv->streams, id) = NULL;
+
+  tp_svc_channel_type_streamed_media_emit_stream_removed (priv->channel, id);
 }
 
 static void priv_stream_ready_cb (SIPMediaStream *stream,
@@ -1564,15 +1624,11 @@ static void priv_stream_supported_codecs_cb (SIPMediaStream *stream,
       /* This remote media description got no codec intersection. */
       switch (priv->state)
         {
-        case SIP_MEDIA_SESSION_STATE_INVITE_SENT:
+        case SIP_MEDIA_SESSION_STATE_RESPONSE_RECEIVED:
         case SIP_MEDIA_SESSION_STATE_INVITE_RECEIVED:
           DEBUG("no codec intersection, closing the stream");
           sip_media_stream_close (stream);
           break;
-        case SIP_MEDIA_SESSION_STATE_REINVITE_SENT:
-          g_warning ("re-INVITE got a response with a non-intersecting codec set");
-          sip_media_stream_close (stream);
-          break;
         case SIP_MEDIA_SESSION_STATE_REINVITE_RECEIVED:
           /* In this case, we have the stream negotiated already,
            * and we don't want to close it just because the remote party
@@ -1580,6 +1636,12 @@ static void priv_stream_supported_codecs_cb (SIPMediaStream *stream,
            * Roll back the whole session to the previously negotiated state. */
           priv_session_rollback (session);
           return;
+        case SIP_MEDIA_SESSION_STATE_ACTIVE:
+          /* We've most likely rolled back from
+           * SIP_MEDIA_SESSION_STATE_REINVITE_RECEIVED,
+           * but we may receive more than one empty codec intersection
+           * in the session, so we ignore the rest */
+          return;
         default:
           g_assert_not_reached();
         }
@@ -1616,28 +1678,26 @@ priv_create_media_stream (SIPMediaSession *self,
                           guint media_type,
                           guint pending_send_flags)
 {
-  SIPMediaSessionPrivate *priv;
+  SIPMediaSessionPrivate *priv = SIP_MEDIA_SESSION_GET_PRIVATE (self);
   gchar *object_path;
   SIPMediaStream *stream = NULL;
-
-  g_assert (SIP_IS_MEDIA_SESSION (self));
+  guint stream_id;
 
   DEBUG ("enter");
 
-  priv = SIP_MEDIA_SESSION_GET_PRIVATE (self);
-
   if (media_type == TP_MEDIA_STREAM_TYPE_AUDIO ||
       media_type == TP_MEDIA_STREAM_TYPE_VIDEO) {
 
+    stream_id = priv->streams->len;
     object_path = g_strdup_printf ("%s/MediaStream%u",
                                    priv->object_path,
-                                   priv->streams->len);
+                                   stream_id);
 
     stream = g_object_new (SIP_TYPE_MEDIA_STREAM,
 			   "media-session", self,
 			   "media-type", media_type,
 			   "object-path", object_path,
-			   "id", priv->streams->len,
+			   "id", stream_id,
                            "pending-send-flags", pending_send_flags,
 			   NULL);
 
@@ -1666,7 +1726,10 @@ priv_create_media_stream (SIPMediaSession *self,
       priv_emit_new_stream (self, stream);
     }
 
-    g_signal_emit (self, signals[SIG_STREAM_ADDED], 0, stream);
+    tp_svc_channel_type_streamed_media_emit_stream_added (priv->channel,
+                                                          stream_id,
+                                                          priv->peer,
+                                                          media_type);
   }
 
   /* note: we add an entry even for unsupported media types */
