@@ -291,6 +291,7 @@ sip_text_channel_set_property(GObject *object,
     priv->object_path = g_value_dup_string (value);
     break;
 
+  case PROP_CHANNEL_TYPE:
   case PROP_HANDLE_TYPE:
     /* this property is writable in the interface, but not actually
      * meaningfully changable on this channel, so we do nothing */
@@ -306,26 +307,27 @@ sip_text_channel_set_property(GObject *object,
   }
 }
 
-void
+static void
 sip_text_channel_dispose(GObject *object)
 {
   SIPTextChannel *self = SIP_TEXT_CHANNEL (object);
   SIPTextChannelPrivate *priv = SIP_TEXT_CHANNEL_GET_PRIVATE (self);
 
-  DEBUG("enter");
-
   if (priv->dispose_has_run)
     return;
 
   priv->dispose_has_run = TRUE;
 
+  if (!priv->closed)
+    sip_text_channel_close (self);
+
   /* release any references held by the object here */
 
   if (G_OBJECT_CLASS (sip_text_channel_parent_class)->dispose)
     G_OBJECT_CLASS (sip_text_channel_parent_class)->dispose (object);
 }
 
-void
+static void
 sip_text_channel_finalize(GObject *object)
 {
   SIPTextChannel *self = SIP_TEXT_CHANNEL (object);
@@ -439,15 +441,22 @@ sip_text_channel_acknowledge_pending_messages(TpSvcChannelTypeText *iface,
 
 
 /**
- * sip_text_channel_close
+ * sip_text_channel_close_async
  *
  * Implements DBus method Close
  * on interface org.freedesktop.Telepathy.Channel
  */
 static void
-sip_text_channel_close (TpSvcChannel *iface, DBusGMethodInvocation *context)
+sip_text_channel_close_async (TpSvcChannel *iface,
+                              DBusGMethodInvocation *context)
+{
+  sip_text_channel_close (SIP_TEXT_CHANNEL(iface));
+  tp_svc_channel_return_from_close (context);
+}
+
+void
+sip_text_channel_close (SIPTextChannel *self)
 {
-  SIPTextChannel *self = SIP_TEXT_CHANNEL(iface);
   SIPTextChannelPrivate *priv;
 
   DEBUG("enter");
@@ -458,11 +467,8 @@ sip_text_channel_close (TpSvcChannel *iface, DBusGMethodInvocation *context)
       priv->closed = TRUE;
       tp_svc_channel_emit_closed (self);
     }
-
-  tp_svc_channel_return_from_close (context);
 }
 
-
 /**
  * sip_text_channel_get_channel_type
  *
@@ -702,9 +708,6 @@ sip_text_channel_emit_message_status(SIPTextChannel *obj,
         case 603:
           send_error = TP_CHANNEL_TEXT_SEND_ERROR_PERMISSION_DENIED;
           break;
-        case 404:
-        case 484:
-        case 485:
         case 604:
           send_error = TP_CHANNEL_TEXT_SEND_ERROR_INVALID_CONTACT;
           break;
@@ -721,8 +724,11 @@ sip_text_channel_emit_message_status(SIPTextChannel *obj,
         case 410:
           send_error = TP_CHANNEL_TEXT_SEND_ERROR_INVALID_CONTACT /* TP_CHANNEL_TEXT_SEND_ERROR_OFFLINE? */;
           break;
+        case 404:
+        case 480:
+          send_error = TP_CHANNEL_TEXT_SEND_ERROR_OFFLINE;
+          break;
         case 413:
-        case 414:
         case 513:
           send_error = TP_CHANNEL_TEXT_SEND_ERROR_TOO_LONG;
           break;
@@ -782,7 +788,7 @@ channel_iface_init(gpointer g_iface, gpointer iface_data)
 
 #define IMPLEMENT(x, suffix) tp_svc_channel_implement_##x (\
     klass, sip_text_channel_##x##suffix)
-  IMPLEMENT(close,);
+  IMPLEMENT(close,_async);
   IMPLEMENT(get_channel_type,);
   IMPLEMENT(get_handle,);
   IMPLEMENT(get_interfaces,);
